"""Basic tests.
"""

import logging
import random

import requests

from ox_herd.core.utils import test_utils


class SimpleTest(test_utils.SelfContainedTest):
    """Run some simple tests.
    """

    def test_good_health_check(self):
        "Verify that health check is OK"

        logging.debug('Doing test_good_health_check on %s', str(self))
        result = requests.get(
            'http://localhost:%i/ox_herd/health_check?token=%s' % (
                self._serverInfo.port, self._serverInfo.health_token))
        self.assertEqual(result.status_code, 200)

    def test_bad_health_check(self):
        "Verify that health check returns 500 if try on non-existing queue"

        logging.debug('Doing test_bad_health_check on %s', str(self))
        queue = 'some_random_bad_queue_%s' % random.randint(0, 1e20)
        params = 'token=%s&check_queues=%s' % (
            self._serverInfo.health_token, queue)
        expect_bad = requests.get(
            'http://localhost:%i/ox_herd/health_check?%s' % (
                self._serverInfo.port, params))
        self.assertEqual(expect_bad.status_code, 500)

    def test_configure_job(self):
        """Test that we can trigger job configuration without error.
        """
        session = requests.session()
        session.post('http://localhost:%i/login' % (
            self._serverInfo.port), dict(zip(
                ['username', 'password'],
                self._serverInfo.stub_user.split(':'))))
        url = 'http://localhost:%i/ox_herd/use_plugin' % self._serverInfo.port
        result = session.get(url, params={
            'plugname': 'ox_herd.core.plugins.example_psutil_plugin',
            'plugcomp': 'CheckCPU'})
        self.assertEqual(result.reason, 'OK')
        self.assertEqual(result.status_code, 200)



"""Core tools for ox_herd flask views.
"""


import logging


def make_form_for_task(my_args):
    """Take an ox_herd plugin instance (or something like it) and make form.

This tries to create a form to configure an ox_herd task.
    """
    maker = getattr(my_args, 'get_flask_form_via_cls', None)
    if maker is None:
        logging.info('Item %s has no %s; trying deprecated %s',
                     my_args, 'get_flask_form_via_cls', 'get_flask_form')
        maker = getattr(my_args, 'get_flask_form', None)
    if maker is None:
        msg = ("Item %s does not inherit from OxPlugin so can't configure.\n"
               "Rewrite that class to inherit from OxPlugin or at least\n"
               "provide a get_flask_form_via_cls method.")
        raise ValueError(msg)
    my_form_cls = maker()
    my_form = my_form_cls(obj=my_args)
    return my_form



"""Module containing some plugin to run pytest.
"""

import os
import shutil
import datetime
import gzip
import logging
import tempfile
import subprocess

import boto3

from ox_herd.core.plugins import base
from ox_herd.core.ox_tasks import OxHerdTask
from ox_herd.core.plugins.awstools_plugin import forms


class OxHerdAWSToolsPlugin(base.OxPlugin):
    """Plugin to provide AWS services for ox_herd
    """

    __blueprint = None

    @classmethod
    def set_bp(cls, my_bp):
        """Set blueprint class variable.
        """
        cls.__blueprint = my_bp

    def get_flask_blueprint(self):
        """Implement as required by OxPlugin."""

        return self.__class__.get_bp()

    @classmethod
    def get_bp(cls):
        """Get blueprint class variable.
        """
        return cls.__blueprint

    def name(self):
        """Implement as required by OxPlugin."""

        return 'awstools_plugin'

    def description(self):
        """Implement as required by OxPlugin."""

        return "Plugin to provide AWS tools for ox_herd."

    def get_components(self):
        return [BackupPostgresToAWS()]


class BackupPostgresToAWS(OxHerdTask, base.OxPluginComponent):
    """Task to backup postgres instance to AWS
    """

    def __init__(self, *args, conn_string=None, prefix=None,
                 bucket_name=None, timeout=1800, **kwargs):
        """Initializer.

        :arg *args:    Argumnets to OxHerdTask.__init__.

        :arg conn_string:  Connection string to database.

        :arg **kwargs:     Keyword arguments to OxHerdTask.__init__.

        """
        OxHerdTask.__init__(self, *args, **kwargs)
        self.conn_string = conn_string
        self.prefix = prefix
        self.bucket_name = bucket_name
        self.timeout = timeout

    @classmethod
    def get_flask_form_via_cls(cls):
        """Get flask form class to enter parameters.
        """
        result = forms.BackupForm
        logging.debug('Providing form %s for cls %s', result, cls)
        return result

    @staticmethod
    def get_conn_string(ox_herd_task):
        """Get connection string from class.

By default we just look for ox_herd_task.conn_string but sub-classes can
override to pull from secret vault or something.
        """
        return ox_herd_task.conn_string

    @classmethod
    def main_call(cls, ox_herd_task):

        rval = cls.backup_pion_db(ox_herd_task)
        return {'return_value': rval}

    @classmethod
    def move_file_to_s3(cls, fname, bucket_name, remote_name, **botokw):
        """Move data in file descriptor to s3.

        :param fname:        Path to file to move.

        :param bucket_name:  String name of S3 bucket. If this starts with
                             '@' then we write to location bucket[1:] (this
                             is useful for testing).

        :param **botokw:  Keyword args for boto (e.g., profile, key, etc.).

        ~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-

        PURPOSE:  Move data in file descriptor to s3.

        """
        if not bucket_name:
            raise ValueError('Invalid bucket_name: "%s"' % str(bucket_name))
        if bucket_name[0] == '@':
            logging.info('Using local file location for bucket "%s"' % str(
                bucket_name))
            remote_name = os.path.join(bucket_name[1:], remote_name)
            os.makedirs(os.path.dirname(remote_name))
            shutil.copy(fname, remote_name)
        else:
            session = boto3.Session(**botokw)
            s3_client = session.client('s3')

            s3_client.upload_file(fname, bucket_name, remote_name)

    @classmethod
    def make_dump_cmdline(cls, ox_herd_task):
        """Make command line to use to dump database.
        """
        return ['pg_dump', '-w', '--dbname=%s' % cls.get_conn_string(
            ox_herd_task)]        
        
    @classmethod
    def backup_pion_db(cls, ox_herd_task):
        """Do main work of backing up database to S3.
        """
        status = None
        written = 0
        remote_name = 'postgres_backups/%s/%s' % (
            ox_herd_task.prefix, datetime.datetime.utcnow().strftime(
                'backup_%A.sql.gz'))

        with tempfile.NamedTemporaryFile(mode='wb') as my_fd:
            logging.info('Using tempfile %s', getattr(my_fd, 'name', '?'))
            zipper = gzip.GzipFile(remote_name, 'wb', 9, my_fd)
            cmd = cls.make_dump_cmdline(ox_herd_task)
            logging.info('Dumping DB to temp file %s and then aws', my_fd.name)
            popen = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                                     stderr=subprocess.PIPE,
                                     universal_newlines=True)
            for stdout_line in iter(popen.stdout.readline, ''):
                written += zipper.write(bytes(stdout_line, 'utf8'))
            popen.stdout.close()
            status = popen.wait(timeout=ox_herd_task.timeout)
            zipper.close()
            if not written:
                raise ValueError('Did not get any backup data')

            cls.move_file_to_s3(my_fd.name, ox_herd_task.bucket_name,
                                remote_name)

        msgs = ['Finished backup succesfully\nStatus=%s\n%s bytes written.' % (
            status, written)]
        msgs.append(popen.stderr.read().strip())
        if msgs[-1]:
            msgs[-1] = 'Errors: %s' % str(msgs[-1])

        return '\n'.join(msgs)
